import { Language, Library } from "../../languages";
/**
 * Enum for generator directories for non-runtime generators
 */
export declare enum OtherGenerators {
    DOCS = "docs",
    TYPESCRIPT_CDK_INFRASTRUCTURE = "typescript-cdk-infrastructure",
    PYTHON_CDK_INFRASTRUCTURE = "python-cdk-infrastructure",
    JAVA_CDK_INFRASTRUCTURE = "java-cdk-infrastructure"
}
/**
 * Generator directory for openapi generation containing templates, config etc.
 */
export type GeneratorDirectory = Language | Library | OtherGenerators;
/**
 * Types of normalizers supported by openapi-generator
 * @see https://openapi-generator.tech/docs/customization/#openapi-normalizer
 */
export type OpenApiNormalizer = "KEEP_ONLY_FIRST_TAG_IN_OPERATION";
/**
 * Options for generating code or docs using OpenAPI Generator CLI
 */
export interface GenerationOptions {
    /**
     * The OpenAPI generator to use to generate the code/docs
     */
    readonly generator: string;
    /**
     * The directory to use for OpenAPI generation
     */
    readonly generatorDirectory: GeneratorDirectory;
    /**
     * The path of the OpenAPI spec to generate code from
     */
    readonly specPath: string;
    /**
     * The directory in which the generated code should be output
     */
    readonly outputPath: string;
    /**
     * Additional properties to pass to the generate cli
     */
    readonly additionalProperties?: {
        [key: string]: string;
    };
    /**
     * Supply the relative path from the code project root to the source code directory in which custom generated files
     * (eg. operation config) should be placed.
     */
    readonly srcDir?: string;
    /**
     * Normalizers to apply to the spec prior to generation, if any
     * @see https://openapi-generator.tech/docs/customization/#openapi-normalizer
     */
    readonly normalizers?: Partial<Record<OpenApiNormalizer, boolean>>;
    /**
     * Vendor extensions to add for code generation, allowing custom properties to be passed to the generator templates
     * Keys should begin with "x-"
     */
    readonly extraVendorExtensions?: Record<string, string>;
}
export interface CommandDetails {
    readonly command: string;
    readonly workingDir: string;
}
/**
 * Generate code or docs by invoking the root generate script
 */
export declare const buildInvokeOpenApiGeneratorCommand: (options: GenerationOptions) => CommandDetails;
/**
 * Builds a command to clean up files which were previously generated by openapi generator
 */
export declare const buildCleanOpenApiGeneratedCodeCommand: (outdir: string) => CommandDetails;
/**
 * Options for generating documentation via a custom generator script
 */
export interface CustomDocsGenerationOptions {
    /**
     * Name of the generator script which exists in scripts/custom/docs
     */
    readonly generator: string;
    /**
     * The path of the OpenAPI spec to generate docs for
     */
    readonly specPath: string;
    /**
     * The directory in which the generated code should be output
     */
    readonly outputPath: string;
}
/**
 * Invoke a custom documentation generator script
 */
export declare const buildInvokeCustomDocsGeneratorCommand: (options: CustomDocsGenerationOptions) => CommandDetails;
